"use client";
// Chakra imports
import {
  Flex,
  Table,
  Tbody,
  Td,
  Text,
  Th,
  Thead,
  Tr,
  useColorModeValue,
  Checkbox,
} from "@chakra-ui/react";

//libraries imports
import React, { useEffect, useMemo, useState } from "react";
import {
  useGlobalFilter,
  usePagination,
  useRowSelect,
  useSortBy,
  useTable,
} from "react-table";

// Components imports
import AddButton from "@/components/buttons/add";
import Card from "@/components/card/Card";
import Pagination from "@/components/pagination/pagination";
import Search from "@/components/search/Search";
import PageSize from "@/components/pagination/PageSize";

// Icon imports
import { DeleteIcon, EditIcon } from "@chakra-ui/icons";

//Data imports
import { TableProps } from "@/variables/columnsData";
import { useRouter, useSearchParams } from "next/navigation";

export default function DataTable(props: TableProps) {
  const {
    addButton = "",
    filterSlot,
    showSearch = true,
    columnsData,
    tableData = [],
    paginate,
    isSelectable = false,
    showPageSize = false,
    onEdit = (id: number) => {},
    onDelete = (id: number) => {},
    onAdd = () => {},
    actionSlot = (index: number | string) => {},
    cellRenderer = (cell: any) => {},
    showActions = true,
    onSelectionChange = () => {},
    pageSize = 10,
  } = props;
  const router = useRouter();
  const searchParams = useSearchParams();

  const [sortBy, setSortBy] = useState<{ id: string; desc: Boolean }>({
    id: searchParams.get("sortyBy") ? searchParams.get("sortyBy") : "",
    desc: searchParams.get("sortType") == "desc" ? true : false,
  }); // Default sort

  const columns = useMemo(() => columnsData, [columnsData]);
  const data = useMemo(() => tableData, [tableData]);

  const tableInstance = useTable(
    {
      columns,
      data,
      initialState: { pageSize: pageSize },
    },
    useGlobalFilter,
    useSortBy,
    usePagination,
    useRowSelect
  );

  const {
    getTableProps,
    getTableBodyProps,
    headerGroups,
    page,
    rows,
    prepareRow,
    setGlobalFilter,
  } = tableInstance;

  const textColor = useColorModeValue("secondaryGray.900", "white");
  const borderColor = useColorModeValue("gray.200", "whiteAlpha.100");

  const [isMounted, setIsMounted] = useState(false);

  useEffect(() => {
    if (isMounted) return;
    setIsMounted(true);
  }, [isMounted]);

  if (!isMounted) return <></>;

  const handleSort = (
    columnId: string,
    sortById: string,
    sortByDesc: Boolean
  ) => {
    setSortBy((prev) => ({
      id: columnId,
      desc: prev.id === columnId ? !prev.desc : false, // Toggle sort direction
    }));
    const sort = sortById === columnId ? !sortByDesc : false;
    const currentParams = new URLSearchParams(searchParams.toString());
    currentParams.set("sortyBy", columnId);
    currentParams.set("sortType", sort ? "desc" : "asc");
    const newUrl = `?${currentParams.toString()}`;
    router.push(newUrl);
  };

  const defaultEdit = (id: number) => (
    <EditIcon color={textColor} h="18px" w="19px" onClick={() => onEdit(id)} />
  );
  const defaultDelete = (id: number) => (
    <DeleteIcon
      color={textColor}
      h="18px"
      w="19px"
      onClick={() => onDelete(id)}
    />
  );

  const defaultActions = (
    id: number,
    actionSlot: (id: number) => JSX.Element | void
  ) => {
    // Ensure actionSlot returns a value (null or JSX)
    const customActions = actionSlot(id);

    // Check if customActions is valid JSX, otherwise return the default icons
    if (customActions) {
      return customActions;
    } else {
      return (
        <>
          {/* {defaultEdit(id)}
          {defaultDelete(id)} */}
        </>
      );
    }
  };

  const defaultRows = (
    cell: any,
    cellRenderer: (id: number) => JSX.Element | void
  ) => {
    const customRows = cellRenderer(cell);
    // Check if customRows is valid JSX, otherwise return the default icons
    if (customRows) {
      return customRows;
    } else {
      return (
        <Text color={textColor} fontSize="sm" fontWeight="700">
          {cell?.render("Cell")}
        </Text>
      );
    }
  };

  const handleRowSelection = (row: any) => {
    const newSelected = !row.isSelected;
    row.toggleRowSelected(newSelected);
    
    // Calculate the new selected rows based on all rows' current state
    if (onSelectionChange) {
      const updatedSelectedRows = rows
        .filter(r => (r === row ? newSelected : r.isSelected))
        .map(r => r.original);
      onSelectionChange(updatedSelectedRows);
    }
  };

  const handleSelectAll = () => {
    const newSelected = !tableInstance.isAllRowsSelected;
    tableInstance.toggleAllRowsSelected(newSelected);
    
    if (onSelectionChange) {
      const updatedSelectedRows = newSelected 
        ? rows.map(r => r.original)
        : [];
      onSelectionChange(updatedSelectedRows);
    }
  };

  return (
    <Card flexDirection="column" w="100%" px="0px">
      {filterSlot && filterSlot}
      <Flex px="25px" justify="space-between" mb="20px" align="center" gap={2}>
        <Flex gap={2}>{addButton && addButton}</Flex>
        <Flex gap={2}>
          {showSearch && (
            <Search
              tableInstance={tableInstance}
              setGlobalFilter={setGlobalFilter}
            />
          )}
          {showPageSize && <PageSize tableInstance={tableInstance} />}
        </Flex>
        {/* <AddButton onClick={onAdd} /> */}
      </Flex>
      <Flex overflowX={"auto"} className="hide-scrollbar">
        {data?.length > 0 ? (
          <Table
            {...getTableProps()}
            variant="simple"
            color="gray.500"
            mb="24px"
          >
            <Thead>
              {headerGroups.map((headerGroup, index) => (
                <Tr {...headerGroup.getHeaderGroupProps()} key={index}>
                  {isSelectable && (
                    <Th pe="10px" key={index} borderColor={borderColor}>
                      <Checkbox
                        onChange={handleSelectAll}
                        isChecked={tableInstance.isAllRowsSelected}
                      />
                    </Th>
                  )}
                  {headerGroup.headers.map((column, index) => (
                    <Th
                      {...column.getHeaderProps()}
                      onClick={() =>
                        handleSort(column.id, sortBy.id, sortBy.desc)
                      }
                      pe="10px"
                      key={index}
                      borderColor={borderColor}
                    >
                      <Flex
                        justify="space-between"
                        align="center"
                        fontSize={{ sm: "10px", lg: "12px" }}
                        color="gray.400"
                      >
                        {column.render("Header")}
                        {sortBy.id === column.id
                          ? sortBy.desc
                            ? " 🔽"
                            : " 🔼"
                          : ""}
                      </Flex>
                    </Th>
                  ))}
                  {showActions && (
                    <Th key={"actions"} pe="10px" borderColor={borderColor}>
                      <Flex
                        justify="space-between"
                        align="center"
                        fontSize={{ sm: "10px", lg: "12px" }}
                        color="gray.400"
                      >
                        Actions
                      </Flex>
                    </Th>
                  )}
                </Tr>
              ))}
            </Thead>

            <Tbody {...getTableBodyProps()}>
              {page?.map((row, index) => {
                prepareRow(row);
                return (
                  <Tr {...row.getRowProps()} key={index}>
                    {isSelectable && (
                      <Td
                        key={index}
                        fontSize={{ sm: "14px" }}
                        minW={{ sm: "150px", md: "200px", lg: "auto" }}
                        borderColor="transparent"
                      >
                        <Checkbox
                          onChange={() => handleRowSelection(row)}
                          isChecked={row.isSelected}
                        />
                      </Td>
                    )}
                    {row.cells.map((cell, index) => {
                      return (
                        <Td
                          {...cell.getCellProps()}
                          key={index}
                          fontSize={{ sm: "14px" }}
                          minW={{ sm: "150px", md: "200px", lg: "auto" }}
                          borderColor="transparent"
                        >
                          {defaultRows(cell, cellRenderer)}
                        </Td>
                      );
                    })}
                    {showActions && (
                    <Td
                      key={index}
                      minW={{ sm: "150px", md: "200px", lg: "auto" }}
                      borderColor="transparent"
                    >
                      {defaultActions(
                        Number((row.original as { id: number }).id),
                        actionSlot
                      )}
                    </Td>)}
                  </Tr>
                );
              })}
            </Tbody>
          </Table>
        ) : (
          <Flex w={"full"} justifyContent={"center"}>
            <Text textAlign={"center"}>No data found</Text>
          </Flex>
        )}
      </Flex>

      {paginate && paginate?.total > 0 && (
        <Pagination tableInstance={paginate} />
      )}
    </Card>
  );
}
